@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -46,7 +46,6 @@
 /* #define DEBUG_UCONTEXT */
 
 NOEXPORT void s_poll_realloc(s_poll_set *);
-NOEXPORT int get_socket_error(const SOCKET);
 
 /**************************************** s_poll functions */
 
@@ -148,6 +147,15 @@ int s_poll_rdhup(s_poll_set *fds, SOCKET fd) {
     return 0; /* not listed in fds */
 }
 
+int s_poll_err(s_poll_set *fds, SOCKET fd) {
+    unsigned i;
+
+    for(i=0; i<fds->nfds; i++)
+        if(fds->ufds[i].fd==fd)
+            return fds->ufds[i].revents&POLLERR;
+    return 0; /* not listed in fds */
+}
+
 NOEXPORT void s_poll_realloc(s_poll_set *fds) {
     fds->ufds=str_realloc(fds->ufds, fds->allocated*sizeof(struct pollfd));
 }
@@ -156,8 +164,8 @@ void s_poll_dump(s_poll_set *fds, int level) {
     unsigned i;
 
     for(i=0; i<fds->nfds; i++)
-        s_log(level, "fd=%d events=0x%X revents=0x%X",
-            fds->ufds[i].fd, fds->ufds[i].events, fds->ufds[i].revents);
+        s_log(level, "FD=%ld events=0x%X revents=0x%X",
+            (long)fds->ufds[i].fd, fds->ufds[i].events, fds->ufds[i].revents);
 }
 
 #ifdef USE_UCONTEXT
@@ -386,23 +394,31 @@ void s_poll_remove(s_poll_set *fds, SOCKET fd) {
 }
 
 int s_poll_canread(s_poll_set *fds, SOCKET fd) {
-    return FD_ISSET(fd, fds->orfds) || FD_ISSET(fd, fds->oxfds);
+    /* ignore exception if there is no error (WinCE 6.0 anomaly) */
+    return FD_ISSET(fd, fds->orfds) ||
+        (FD_ISSET(fd, fds->oxfds) && get_socket_error(fd));
 }
 
 int s_poll_canwrite(s_poll_set *fds, SOCKET fd) {
-    return FD_ISSET(fd, fds->owfds) || FD_ISSET(fd, fds->oxfds);
+    /* ignore exception if there is no error (WinCE 6.0 anomaly) */
+    return FD_ISSET(fd, fds->owfds) ||
+        (FD_ISSET(fd, fds->oxfds) && get_socket_error(fd));
 }
 
 int s_poll_hup(s_poll_set *fds, SOCKET fd) {
     (void)fds; /* squash the unused parameter warning */
     (void)fd; /* squash the unused parameter warning */
-    return 0; /* FIXME: how to detect HUP condition with select()? */
+    return 0; /* FIXME: how to detect the HUP condition with select()? */
 }
 
 int s_poll_rdhup(s_poll_set *fds, SOCKET fd) {
     (void)fds; /* squash the unused parameter warning */
     (void)fd; /* squash the unused parameter warning */
-    return 0; /* FIXME: how to detect RDHUP condition with select()? */
+    return 0; /* FIXME: how to detect the RDHUP condition with select()? */
+}
+
+int s_poll_err(s_poll_set *fds, SOCKET fd) {
+    return FD_ISSET(fd, fds->oxfds);
 }
 
 #ifdef USE_WIN32
@@ -453,7 +469,7 @@ void s_poll_dump(s_poll_set *fds, int level) {
         ow=FD_ISSET(fd, fds->owfds);
         ox=FD_ISSET(fd, fds->oxfds);
         if(ir || iw || ix || or || ow || ox)
-            s_log(level, "fd=%d ifds=%c%c%c ofds=%c%c%c", fd,
+            s_log(level, "FD=%ld ifds=%c%c%c ofds=%c%c%c", (long)fd,
                 ir?'r':'-', iw?'w':'-', ix?'x':'-',
                 or?'r':'-', ow?'w':'-', ox?'x':'-');
     }
@@ -508,7 +524,7 @@ int set_socket_options(SOCKET s, int type) {
     return retval; /* returns 0 when all options succeeded */
 }
 
-NOEXPORT int get_socket_error(const SOCKET fd) {
+int get_socket_error(const SOCKET fd) {
     int err;
     socklen_t optlen=sizeof err;
 
@@ -651,9 +667,8 @@ void fd_putline(CLI *c, SOCKET fd, const char *line) {
     tmpline=str_printf("%s%s", line, crlf);
     len=strlen(tmpline);
     s_write(c, fd, tmpline, len);
-    tmpline[len-2]='\0'; /* remove CRLF */
-    s_log(LOG_DEBUG, " -> %s", tmpline);
     str_free(tmpline);
+    s_log(LOG_DEBUG, " -> %s", line);
 }
 
 char *fd_getline(CLI *c, SOCKET fd) {
@@ -776,7 +791,7 @@ char *ssl_getstring(CLI *c) { /* get null-terminated string */
     line=str_alloc(allocated);
     for(;;) {
         if(ptr>65536) { /* >64KB --> DoS protection */
-            s_log(LOG_ERR, "fd_getline: Line too long");
+            s_log(LOG_ERR, "ssl_getstring: Line too long");
             str_free(line);
             longjmp(c->err, 1);
         }
@@ -792,6 +807,52 @@ char *ssl_getstring(CLI *c) { /* get null-terminated string */
     return line;
 }
 
+char *ssl_getline(CLI *c) { /* get newline-terminated string */
+    char *line;
+    size_t ptr=0, allocated=32;
+
+    line=str_alloc(allocated);
+    for(;;) {
+        if(ptr>65536) { /* >64KB --> DoS protection */
+            s_log(LOG_ERR, "ssl_getline: Line too long");
+            str_free(line);
+            longjmp(c->err, 1);
+        }
+        if(allocated<ptr+1) {
+            allocated*=2;
+            line=str_realloc(line, allocated);
+        }
+        s_ssl_read(c, line+ptr, 1);
+        if(line[ptr]=='\r')
+            continue;
+        if(line[ptr]=='\n')
+            break;
+        if(line[ptr]=='\0')
+            break;
+        ++ptr;
+    }
+    line[ptr]='\0';
+    s_log(LOG_DEBUG, " <- %s", line);
+    return line;
+}
+
+void ssl_putline(CLI *c, const char *line) { /* put newline-terminated string */
+    char *tmpline;
+    const char crlf[]="\r\n";
+    size_t len;
+
+    tmpline=str_printf("%s%s", line, crlf);
+    len=strlen(tmpline);
+    if(len>INT_MAX) { /* paranoia */
+        s_log(LOG_ERR, "ssl_putline: Line too long");
+        str_free(tmpline);
+        longjmp(c->err, 1);
+    }
+    s_ssl_write(c, tmpline, (int)len);
+    str_free(tmpline);
+    s_log(LOG_DEBUG, " -> %s", line);
+}
+
 /**************************************** network helpers */
 
 #define INET_SOCKET_PAIR
