@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -217,9 +217,9 @@ typedef struct service_options_struct {
     char *cert;                                             /* cert filename */
     char *key;                               /* pem (priv key/cert) filename */
     long session_size, session_timeout;
-    long ssl_options_set;
+    long unsigned ssl_options_set;
 #if OPENSSL_VERSION_NUMBER>=0x009080dfL
-    long ssl_options_clear;
+    long unsigned ssl_options_clear;
 #endif /* OpenSSL 0.9.8m or later */
     SSL_METHOD *client_method, *server_method;
     SOCKADDR_UNION sessiond_addr;
@@ -428,7 +428,7 @@ void stunnel_info(int);
 
 /**************************************** prototypes for options.c */
 
-extern char *configuration_file;
+extern char configuration_file[PATH_MAX];
 
 int options_cmdline(char *, char *);
 int options_parse(CONF_TYPE);
@@ -510,6 +510,7 @@ void sslerror(char *);
 /**************************************** prototypes for verify.c */
 
 int verify_init(SERVICE_OPTIONS *);
+void print_client_CA_list(const STACK_OF(X509_NAME) *);
 char *X509_NAME2text(X509_NAME *);
 
 /**************************************** prototypes for network.c */
@@ -523,6 +524,7 @@ int s_poll_canread(s_poll_set *, SOCKET);
 int s_poll_canwrite(s_poll_set *, SOCKET);
 int s_poll_hup(s_poll_set *, SOCKET);
 int s_poll_rdhup(s_poll_set *, SOCKET);
+int s_poll_err(s_poll_set *, SOCKET);
 int s_poll_wait(s_poll_set *, int, int);
 void s_poll_dump(s_poll_set *, int);
 
@@ -548,6 +550,7 @@ void client_main(CLI *);
 
 /**************************************** prototypes for network.c */
 
+int get_socket_error(const SOCKET);
 int s_connect(CLI *, SOCKADDR_UNION *, socklen_t);
 void s_write(CLI *, SOCKET fd, const void *, size_t);
 void s_read(CLI *, SOCKET fd, void *, size_t);
@@ -630,21 +633,20 @@ int getnameinfo(const struct sockaddr *, socklen_t,
 /**************************************** prototypes for sthreads.c */
 
 typedef enum {
-    CRIT_SESSION, CRIT_ADDR,
-    CRIT_CLIENTS, CRIT_SSL,                 /* client.c */
-    CRIT_INET,                              /* resolver.c */
+    LOCK_SESSION, LOCK_ADDR,
+    LOCK_CLIENTS, LOCK_SSL,                 /* client.c */
+    LOCK_INET,                              /* resolver.c */
 #ifndef USE_WIN32
-    CRIT_LIBWRAP,                           /* libwrap.c */
+    LOCK_LIBWRAP,                           /* libwrap.c */
 #endif
-    CRIT_LOG, CRIT_LEAK,                    /* log.c */
+    LOCK_LOG, LOCK_LEAK,                    /* log.c */
 #ifndef OPENSSL_NO_DH
-    CRIT_DH,                                /* ctx.c */
+    LOCK_DH,                                /* ctx.c */
 #endif /* OPENSSL_NO_DH */
-    CRIT_SECTIONS                           /* number of critical sections */
-} SECTION_CODE;
+    STUNNEL_LOCKS                           /* number of locks */
+} LOCK_TYPE;
+extern int stunnel_locks[STUNNEL_LOCKS];
 
-void enter_critical_section(SECTION_CODE);
-void leave_critical_section(SECTION_CODE);
 int sthreads_init(void);
 unsigned long stunnel_process_id(void);
 unsigned long stunnel_thread_id(void);
