@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -38,7 +38,7 @@
 #include "common.h"
 #include "prototypes.h"
 
-    /* global OpenSSL initalization: compression, engine, entropy */
+    /* global OpenSSL initialization: compression, engine, entropy */
 NOEXPORT void cb_free(void *parent, void *ptr, CRYPTO_EX_DATA *ad,
     int idx, long argl, void *argp);
 #ifndef OPENSSL_NO_COMP
@@ -127,7 +127,7 @@ NOEXPORT int compression_init(GLOBAL_OPTIONS *global) {
     }
 
     if(global->compression==COMP_NONE ||
-            SSLeay()<0x00908051L /* 0.9.8e-beta1 */) {
+            OpenSSL_version_num()<0x00908051L /* 0.9.8e-beta1 */) {
         /* delete OpenSSL defaults (empty the SSL_COMP stack) */
         /* cannot use sk_SSL_COMP_pop_free,
          * as it also destroys the stack itself */
@@ -172,9 +172,6 @@ NOEXPORT int compression_init(GLOBAL_OPTIONS *global) {
 NOEXPORT int prng_init(GLOBAL_OPTIONS *global) {
     int totbytes=0;
     char filename[256];
-#ifndef USE_WIN32
-    int bytes;
-#endif
 
     filename[0]='\0';
 
@@ -207,9 +204,11 @@ NOEXPORT int prng_init(GLOBAL_OPTIONS *global) {
         return 0; /* success */
     }
     s_log(LOG_DEBUG, "RAND_screen failed to sufficiently seed PRNG");
-#elif !defined(OPENSSL_NO_EGD)
+#else
+#ifndef OPENSSL_NO_EGD
     if(global->egd_sock) {
-        if((bytes=RAND_egd(global->egd_sock))==-1) {
+        int bytes=RAND_egd(global->egd_sock);
+        if(bytes==-1) {
             s_log(LOG_WARNING, "EGD Socket %s failed", global->egd_sock);
             bytes=0;
         } else {
@@ -220,7 +219,7 @@ NOEXPORT int prng_init(GLOBAL_OPTIONS *global) {
                          so no need to check if seeded sufficiently */
         }
     }
-#else
+#endif
     /* try the good-old default /dev/urandom, if available  */
     totbytes+=add_rand_file(global, "/dev/urandom");
     if(RAND_status())
