@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -40,18 +40,18 @@
 
 /* http://www.openssl.org/support/faq.html#PROG2 */
 #ifdef USE_WIN32
+
 #ifdef __GNUC__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-pedantic"
 #endif /* __GNUC__ */
-#ifdef __GNUC__
-#include <../ms/applink.c>
-#else /* __GNUC__ */
+
 #include <openssl/applink.c>
-#endif /* __GNUC__ */
+
 #ifdef __GNUC__
 #pragma GCC diagnostic pop
 #endif /* __GNUC__ */
+
 #endif /* USE_WIN32 */
 
 /**************************************** prototypes */
@@ -125,10 +125,19 @@ void main_init() { /* one-time initialization */
     stunnel_info(LOG_NOTICE);
 }
 
+/* return values:
+   0 - configuration accepted
+   1 - error
+   2 - information printed
+*/
+
     /* configuration-dependent initialization */
 int main_configure(char *arg1, char *arg2) {
-    if(options_cmdline(arg1, arg2))
-        return 1;
+    int cmdline_status;
+
+    cmdline_status=options_cmdline(arg1, arg2);
+    if(cmdline_status) /* cannot proceed */
+        return cmdline_status;
     options_apply();
     str_canary_init(); /* needs prng initialization from options_cmdline */
 #if !defined(USE_WIN32) && !defined(__vms)
@@ -330,8 +339,8 @@ NOEXPORT int accept_connection(SERVICE_OPTIONS *opt) {
         }
     }
     from_address=s_ntop(&addr, addrlen);
-    s_log(LOG_DEBUG, "Service [%s] accepted (FD=%d) from %s",
-        opt->servname, s, from_address);
+    s_log(LOG_DEBUG, "Service [%s] accepted (FD=%ld) from %s",
+        opt->servname, (long)s, from_address);
     str_free(from_address);
 #ifdef USE_FORK
     RAND_add("", 1, 0.0); /* each child needs a unique entropy pool */
@@ -370,8 +379,8 @@ void unbind_ports(void) {
             if(opt->fd<(SOCKET)listen_fds_start ||
                     opt->fd>=(SOCKET)(listen_fds_start+systemd_fds))
                 closesocket(opt->fd);
-            s_log(LOG_DEBUG, "Service [%s] closed (FD=%d)",
-                opt->servname, opt->fd);
+            s_log(LOG_DEBUG, "Service [%s] closed (FD=%ld)",
+                opt->servname, (long)opt->fd);
             opt->fd=INVALID_SOCKET;
 #ifdef HAVE_STRUCT_SOCKADDR_UN
             if(opt->local_addr.sa.sa_family==AF_UNIX) {
@@ -435,15 +444,15 @@ int bind_ports(void) {
             if(listening_section<systemd_fds) {
                 opt->fd=(SOCKET)(listen_fds_start+listening_section);
                 s_log(LOG_DEBUG,
-                    "Listening file descriptor received from systemd (FD=%d)",
-                    opt->fd);
+                    "Listening file descriptor received from systemd (FD=%ld)",
+                    (long)opt->fd);
             } else {
                 opt->fd=s_socket(opt->local_addr.sa.sa_family,
                     SOCK_STREAM, 0, 1, "accept socket");
                 if(opt->fd==INVALID_SOCKET)
                     return 1;
-                s_log(LOG_DEBUG, "Listening file descriptor created (FD=%d)",
-                    opt->fd);
+                s_log(LOG_DEBUG, "Listening file descriptor created (FD=%ld)",
+                    (long)opt->fd);
             }
             if(set_socket_options(opt->fd, 0)<0) {
                 closesocket(opt->fd);
@@ -478,22 +487,25 @@ int bind_ports(void) {
                 /* fchown() does *not* work on UNIX sockets */
                 if(!lchown(opt->local_addr.un.sun_path, opt->uid, opt->gid))
                     s_log(LOG_DEBUG,
-                        "Socket chown succeeded: %s, UID=%d, GID=%d",
-                        opt->local_addr.un.sun_path, opt->uid, opt->gid);
+                        "Socket chown succeeded: %s, UID=%u, GID=%u",
+                        opt->local_addr.un.sun_path,
+                        (unsigned)opt->uid, (unsigned)opt->gid);
                 else if(lstat(opt->local_addr.un.sun_path, &sb))
                     sockerror(opt->local_addr.un.sun_path);
                 else if(sb.st_uid==opt->uid && sb.st_gid==opt->gid)
                     s_log(LOG_DEBUG,
-                        "Socket chown unneeded: %s, UID=%d, GID=%d",
-                        opt->local_addr.un.sun_path, opt->uid, opt->gid);
+                        "Socket chown unneeded: %s, UID=%u, GID=%u",
+                        opt->local_addr.un.sun_path,
+                        (unsigned)opt->uid, (unsigned)opt->gid);
                 else
-                    s_log(LOG_ERR, "Socket chown failed: %s, UID=%d, GID=%d",
-                        opt->local_addr.un.sun_path, opt->uid, opt->gid);
+                    s_log(LOG_ERR, "Socket chown failed: %s, UID=%u, GID=%u",
+                        opt->local_addr.un.sun_path,
+                        (unsigned)opt->uid, (unsigned)opt->gid);
             }
 #endif
             s_poll_add(fds, opt->fd, 1, 0);
-            s_log(LOG_DEBUG, "Service [%s] (FD=%d) bound to %s",
-                opt->servname, opt->fd, local_address);
+            s_log(LOG_DEBUG, "Service [%s] (FD=%ld) bound to %s",
+                opt->servname, (long)opt->fd, local_address);
             str_free(local_address);
             ++listening_section;
         } else if(opt->exec_name && opt->connect_addr.names) {
@@ -782,9 +794,9 @@ NOEXPORT char *signal_name(int signum) {
 
 void stunnel_info(int level) {
     s_log(level, "stunnel " STUNNEL_VERSION " on " HOST " platform");
-    if(strcmp(OPENSSL_VERSION_TEXT, SSLeay_version(SSLEAY_VERSION))) {
+    if(strcmp(OPENSSL_VERSION_TEXT, OpenSSL_version(OPENSSL_VERSION))) {
         s_log(level, "Compiled with " OPENSSL_VERSION_TEXT);
-        s_log(level, "Running  with %s", SSLeay_version(SSLEAY_VERSION));
+        s_log(level, "Running  with %s", OpenSSL_version(OPENSSL_VERSION));
         s_log(level, "Update OpenSSL shared libraries or rebuild stunnel");
     } else {
         s_log(level, "Compiled/running with " OPENSSL_VERSION_TEXT);
