@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -103,20 +103,14 @@ int verify_init(SERVICE_OPTIONS *section) {
 /* trusted CA names sent to clients for client cert selection */
 NOEXPORT void set_client_CA_list(SERVICE_OPTIONS *section) {
     STACK_OF(X509_NAME) *ca_dn;
-    char *ca_name;
-    int i;
 
     s_log(LOG_DEBUG, "Client CA list: %s", section->ca_file);
     ca_dn=SSL_load_client_CA_file(section->ca_file);
-    for(i=0; i<sk_X509_NAME_num(ca_dn); ++i) {
-        ca_name=X509_NAME2text(sk_X509_NAME_value(ca_dn, i));
-        s_log(LOG_INFO, "Client CA: %s", ca_name);
-        str_free(ca_name);
-    }
     SSL_CTX_set_client_CA_list(section->ctx, ca_dn);
+    print_client_CA_list(ca_dn);
 }
 
-int crl_init(SERVICE_OPTIONS *section) {
+NOEXPORT int crl_init(SERVICE_OPTIONS *section) {
     X509_STORE *store;
 
     store=SSL_CTX_get_cert_store(section->ctx);
@@ -393,13 +387,17 @@ NOEXPORT int compare_pubkeys(X509 *c1, X509 *c2) {
 
 #ifndef OPENSSL_NO_OCSP
 
-/* type checks not available -- use generic functions */
+#ifdef DEFINE_STACK_OF
+/* defined in openssl/safestack.h:
+ * DEFINE_SPECIAL_STACK_OF(OPENSSL_STRING, char) */
+#else /* DEFINE_STACK_OF */
 #ifndef sk_OPENSSL_STRING_num
 #define sk_OPENSSL_STRING_num(st) sk_num(st)
-#endif
+#endif /* sk_OPENSSL_STRING_num */
 #ifndef sk_OPENSSL_STRING_value
 #define sk_OPENSSL_STRING_value(st, i) sk_value((st),(i))
-#endif
+#endif /* sk_OPENSSL_STRING_value */
+#endif /* DEFINE_STACK_OF */
 
 NOEXPORT int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
     X509 *cert;
@@ -709,6 +707,26 @@ NOEXPORT void log_time(const int level, const char *txt, ASN1_TIME *t) {
 
 #endif /* !defined(OPENSSL_NO_OCSP) */
 
+void print_client_CA_list(const STACK_OF(X509_NAME) *ca_dn) {
+    char *ca_name;
+    int n, i;
+
+    if(!ca_dn) {
+        s_log(LOG_INFO, "No client CA list");
+        return;
+    }
+    n=sk_X509_NAME_num(ca_dn);
+    if(n==0) {
+        s_log(LOG_INFO, "Empty client CA list");
+        return;
+    }
+    for(i=0; i<n; ++i) {
+        ca_name=X509_NAME2text(sk_X509_NAME_value(ca_dn, i));
+        s_log(LOG_INFO, "Client CA: %s", ca_name);
+        str_free(ca_name);
+    }
+}
+
 char *X509_NAME2text(X509_NAME *name) {
     char *text;
     BIO *bio;
